#include "Bluetooth.h"
#include "Bluetooth.h"

Bluetooth::Bluetooth(uint8_t type, uint8_t rx, uint8_t tx){
    static SoftwareSerial bluetoothSerial(rx,tx);
    mSerial = &bluetoothSerial;
    serialMonitor = &Serial;
    _type = type;
}

Bluetooth::Bluetooth(uint8_t type, uint8_t rx, uint8_t tx, uint8_t en){
    static SoftwareSerial bluetoothSerial(rx,tx);
    mSerial = &bluetoothSerial;
    serialMonitor = &Serial; 
    _en = en;
    _type = type;
    if(_type == HC05 ) pinMode(_en, OUTPUT);
}

void Bluetooth::startConfig(){
    if(_type == HC05) digitalWrite(_en, HIGH);
    unsigned long timer;
    for(byte i = 0 ; i < 5 ; i++){
        timer = millis();
        serialMonitor -> print(".");
        mSerial -> begin(BAUD[i]);
        delay(50);
        while(millis() - timer < TIME_OUT){
            switch (_type){
                case HC05 : mSerial -> print("AT\r\n");
                    delay(300);
                    break;
                case HC06 : mSerial -> print("AT");
                    delay(600);
                    break;
            }
            if(mSerial -> available() >0){
                if(respone(false).indexOf("OK") != -1) {
                    serialMonitor -> println("\nBluetooth in AT Mode!");
                    serialMonitor -> println("Ready");
                    AT_MODE = true;
                    current_baudrate = BAUD[i];
                    break;
                }
            }
        }
        if(AT_MODE) break;
    }
    if(!AT_MODE) {
        serialMonitor -> println("\nTry Again after 5 seconds");
        delay(5000);
        asm volatile ("jmp 0"); 
    }
}

void Bluetooth::endConfig(){
    if(AT_MODE){
      serialMonitor -> println("Exit, AT Mode !");
        switch (_type){
            case HC05 : digitalWrite(_en,LOW);
                sendAT("AT+RESET");
                break;
            case HC06 :
                break;
        }
        if(current_baudrate != new_baudrate && new_baudrate != 0){
            mSerial -> begin(new_baudrate);
        }
        AT_MODE = false;
    }
}

void Bluetooth::setName(String name){
    serialMonitor -> print("Set Name : ");
    serialMonitor -> println(name);
    if(AT_MODE){
        switch (_type){
            case HC05 : name = "AT+NAME=" + name;
                break;
            case HC06 : name = "AT+NAME" +  name;
                break;
        }
        sendAT(name); 
    }
    else {
        serialMonitor -> println("ERROR");
        serialMonitor -> println("Please config in AT MODE");
    }
}

void Bluetooth::setPassword(String password){
    serialMonitor -> print("Set password : ");
    serialMonitor -> println(password);
    if(AT_MODE){
        switch (_type){
            case HC05: password = "AT+PSWD=" + password;
                break;
            case HC06: password = "AT+PIN" + password;
                break;
        }
        sendAT(password); 
    }
    else {
        serialMonitor -> println("ERROR");
        serialMonitor -> println("Please config in AT MODE");
    }
}

void Bluetooth::begin(long baudrate){
  mSerial -> begin(baudrate);
}

void Bluetooth::setBaudrate(long baudrate){
   serialMonitor -> print("Set Baudrate : ");
   serialMonitor -> println(baudrate);
   if(AT_MODE) {
        if(baudrate != current_baudrate){
            new_baudrate = baudrate;
            if(_type == HC05){
                switch(baudrate){
                case BAUD4: sendAT("AT+UART=9600,0,0");
                    break;
                case BAUD5: sendAT("AT+UART=19200,0,0");
                    break;
                case BAUD6: sendAT("AT+UART=38400,0,0");
                    break;
                case BAUD7: sendAT("AT+UART=57600,0,0");
                    break;
                case BAUD8: sendAT("AT+UART=115200,0,0");
                    break;
                default: serialMonitor -> println("ERROR");
                    break;
                }
            }
            if(_type == HC06){
                switch(baudrate){
                    case BAUD4: sendAT("AT+BAUD4");
                        break;
                    case BAUD5: sendAT("AT+BAUD5");
                        break;
                    case BAUD6: sendAT("AT+BAUD6");
                        break;
                    case BAUD7: sendAT("AT+BAUD7");
                        break;
                    case BAUD8: sendAT("AT+BAUD8");
                        break;
                    default: serialMonitor -> println("ERROR");
                        break;
                }
            }
        } else serialMonitor -> println("OK");   
   }
   else {
    serialMonitor -> println("ERROR");
    serialMonitor -> println("Please config in AT MODE");
   }
}

void Bluetooth::sendData(String cmd){
    mSerial -> print(cmd);
    if(_type == HC05) mSerial -> print("\r\n");
    mSerial -> flush();
}

void Bluetooth::sendAT(String cmd){
    if(AT_MODE){
        mSerial -> print(cmd);
        if(_type == HC05) mSerial -> print("\r\n");
        mSerial -> flush();
        respone(true);
    }
    else {
        serialMonitor -> println("ERROR");
        serialMonitor -> println("Please config in AT MODE");
    }
}

String Bluetooth::getData(){
    data = "";
    while(mSerial -> available() > 0 ){
        data += (char) mSerial -> read();
    }
    return data;
}

String Bluetooth::respone(boolean debug){
    String _respone = "";
    while(true){
        if(mSerial -> available() > 0) {
        _respone = getData();
        if(debug) serialMonitor -> println(_respone);
        break;
        }
    }
    return _respone;
}
